
/*
 * $Id: Opt.h,v 1.1.1.3 1999/03/08 07:56:12 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef OPT_CLASS_H
#define OPT_CLASS_H

#include "String.h"
#include "Array.h"


class Opt;
class CmdLine;

class OptGrp: public Array<Opt*> {
	public:
		virtual ~OptGrp() {}

		virtual ostream &printAnonym(ostream &os) const { return os; }
		virtual bool parseAnonym(const Array<const char *> &) { return false; }
		virtual bool canParseAnonym() const { return false; }

		virtual bool validate() const { return true; }

		void share(OptGrp *other);
};

// generic option
class Opt {
	public:
		Opt(OptGrp *aGrp, const char *aName, const char *aDescr);
		virtual ~Opt();

		void cmdLine(CmdLine *aCmdLine) { theCmdLine = aCmdLine; }

		virtual bool parse(const char *name, const char *val) = 0;
		virtual void report() const = 0;
		virtual bool visible() const { return true; } // use in reports

		const char *name() const { return theName; }
		const char *type() const { return theType; }
		const char *descr() const { return theDescr; }

	protected:
		OptGrp *theGrp;
		CmdLine *theCmdLine;
		String theName;
		String theType;
		String theDescr;
};


// command line parser
class CmdLine {
	public:
		CmdLine(const Array<OptGrp*> &groups);
		virtual ~CmdLine();

		bool parse(int argc, char *argv[]);

		void usage(ostream &os) const;
		void report() const;

	protected:
		// parse one option at a time
		bool parse(const char *name, const char *val);

	protected:
		Array<char *> theArgs;
		Array<Opt *> theOpts;
		OptGrp *theAnonymParser; // group to use for anonymous options

		String thePrgName;   // program name (argv[0])
};

#endif
