
/*
 * $Id: loop.cc,v 1.2 1999/04/15 20:36:00 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "Clock.h"
#include "BitMask.h"
#include "Rnd.h"
#include "StatPing.h"
#include "StatSoft.h"
#include "StatPhaseMgr.h"
#include "World.h"
#include "HitModel.h"
#include "ChbModel.h"
#include "ZipfPopMdl.h"

#include "gadgets.h"
#include "globals.h"

bool ShutdownNow() { Assert(0); return false; }  // XXX: ARGH!

static ChbModel TheChbModel;

static const TheMaxOid = 10000000;
static BitMask TheCachableOids;
static BitMask TheUnCachableOids;
static RndGen TheGen;

static double TheCR = 0.80; // cachablility ratio
static double TheHR = 0.55; // hit ratio

static struct {
	int partCnt;
	int totCnt;
} TheHitCnt, TheChbCnt, TheZeroCnt;

static
void step() {
	OidRec rec;
	const int oid = TheWorld.genOid(rec);
	if (oid > TheMaxOid)
		exit(-1);

	//static int count = 0;
	//cerr << here << setw(7) << ++count << ' ' << setw(7) << oid << endl;

	rec.ohit = rec.repeat && rec.cachable; // ideal proxy
	if (TheCachableOids[oid]) { // hit
		Assert(rec.repeat);
		TheHitCnt.partCnt++;

		/*TheChbCnt.partCnt++; // Client Side CR!
		TheChbCnt.totCnt++; // Client Side CR!
		TheChbModel.gotOid(true); // Client Side CR!
		*/
	} else
	if (TheUnCachableOids[oid]) { // uncachable repeat
		Assert(rec.repeat);
		//TheChbModel.gotOid(false);
		TheChbCnt.totCnt++;
	} else { // first timer
		Assert(!rec.repeat);
		//const bool cachable = TheChbModel.cachableOid();
		rec.cachable ? TheCachableOids.beSet(oid) : TheUnCachableOids.beSet(oid);
		//TheChbModel.gotOid(cachable);

		if (rec.cachable)
			TheChbCnt.partCnt++;
		TheChbCnt.totCnt++;
	}
	TheWorld.noteResponse(rec);
	TheHitCnt.totCnt++;
}

static
void configureLogs(int prec) {
	configureStream(cout, prec);
	configureStream(cerr, prec);
	configureStream(clog, prec);
}

// set some general stuff and
// propogate cmd line options to corresponding objects
static
void configure() {

	configureLogs(3);

	// configure the World
	// XXX: remove crap
	ZipfPopMdl *z = new ZipfPopMdl();
	TheWorld.thePopMdl = z;

	TheWorld.theHitMdl->hr(TheHR);

	TheWorld.theChbMdl->ratios(TheCR, TheHR);

	TheCachableOids.resize(TheMaxOid+1);
	TheUnCachableOids.resize(TheMaxOid+1);
}

int main(int , char *argv[]) {

	/*CmdLine cmd(Array<OptGrp*>() << &TheOpts << &TheCltOpts);
	if (!cmd.parse(argc, argv) || !TheOpts.validate() || !TheCltOpts.validate())
		return -1;*/

	configure();

	//cmd.report();

	TheStatPhaseMgr.goal(Goal());
	Clock::Update();
	ThePinger.start();
	TheSoOftenPinger.start();

	const int ping = 10000;
	int count = atoi(argv[1]);
	for (int i = 1; count < 0 || i <= count; ++i) {
		/*Clock::Update();
		ThePinger.checkpoint();
		TheSoOftenPinger.checkpoint();*/
		step();

		if (i % ping == 0) {
			cout << setw(6) << i/1000
				<< ' ' << setw(7) << Ratio(TheChbCnt.partCnt, TheChbCnt.totCnt)
				<< ' ' << setw(6) << TheHitCnt.partCnt
				<< ' ' << setw(7) << Ratio(TheHitCnt.partCnt, TheHitCnt.totCnt)
				<< endl;
		}

		if (i % ping == 0) {
			TheChbCnt = TheHitCnt = TheZeroCnt;
		}
	}

	//Clock::Update();
	//TheStatPhaseMgr.forceFlush();

	return 0;
}
