
/*
 * $Id: polyOpts.cc,v 1.3.2.4 1999/05/21 04:20:00 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "rndDistrs.h"
#include "LogComment.h"
#include "opts.h"
#include "polyOpts.h"
#include "polyVersion.h"
#include "gadgets.h"
#include "globals.h"


/* Goal */

GoalOpt::GoalOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr), theVal(*this) {
}

bool GoalOpt::parse(const char *name, const char *val) {
	if (val) {
		String s[3], tail;
		int count = 1;

		if (splitVal(val, s[0], tail)) {
			count = 2;
			if (splitVal(tail, s[1], s[2]))
				count = 3;
			else
				s[1] = tail;
		} else 
			s[0] = val;

		int dcount = 0;
		int icount = 0;
		bool valid = true;
		// first integer if any is compl count
		// second integer if any is err count
		// time if any is duration
		for (int i = 0; valid && i < count; ++i) {
			const char *ptr;
			if (isTime(s[i], theDuration))
				valid = !dcount++;
			else
			if (!icount++)
				valid = isInt(s[i], theXactComplCnt, &ptr) && !*ptr;
			else
			if (icount == 2) {
				int h;
				if (isInt(s[i], h,  &ptr) && !*ptr)
					theErrs.theCnt = h;
				else
					valid = isRatio(s[i], theErrs.theRatio);
			} else
				valid = false;
		}
		if (valid)
			return true;
	}
	cerr << ThePrgName << ": goal value (e.g. `1000:1hr:10') expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void GoalOpt::report() const {
	Comment << *this; // theXactComplCnt << ':' << theDuration << ':' << theErrCnt;
}


/* ObjLcOpt */

ObjLcOpt::ObjLcOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr), theVal(*this) {
	// careful with defaults: people may get theShowExpRatio > 100%...
}

// percent = ('now'|'lmt') '+' distribution
String ObjLcOpt::doParse(const char *name, const char *val) {
	if (!val)
		return "no value specified";

	String head, tail;
	double ratio;

	// isolate percent
	if (!splitVal(val, head, tail, '='))
		return "`=' not found";

	if (!isRatio(head, ratio))
		return "percent/ratio token is malformed";

	String model, distr;
	if (!splitVal(tail, model, distr, '+'))
		return "`+' not found";

	RndDistr *d = 0;
	if (model == "now") {
		if (distr == "none" && ratio == 0)
			; // do nothing
		else
		if (!isDistr(name, distr, d, "time"))
			return "distribution token is malformed or unknown";
		nowExpTime(d, ratio);
	} else
	if (model == "lmt") {
		if (distr == "none" && ratio == 0)
			; // do nothing
		else
		if (!isDistr(name, distr, d, "num"))
			return "distribution token is malformed or unknown";
		modExpTime(d, ratio);
	} else
		return "expiration model is unknown (not `now' or `lmt')";

	if (theShowExpRatio > 1) {
		cerr << ThePrgName << ": percent for `now' model:  " << 100*theNowExpRatio << endl;
		cerr << ThePrgName << ": percent for `lmt' model:  " << 100*theModExpRatio << endl;
		cerr << ThePrgName << ": total percent for models: " << 100*theShowExpRatio << endl;
		return "total percent for two models is greater than 100%";
	}

	return 0; // no errors
}

bool ObjLcOpt::parse(const char *name, const char *val) {
	if (const String err = doParse(name, val)) {
		cerr << ThePrgName << ": obj expiration model (e.g. `50p=now+exp:1hour' or `27p=lmt+const:1') expected for the `" << name << "` option; got `" << valStr(val) << "'"
			<< "; it is possible that " << err << endl;
		return false;
	}
	return true;
}

void ObjLcOpt::report() const {
	Comment	<< (100*theNowExpRatio) << "%=now+";
	if (theNowExpTime) 
		Comment << *theNowExpTime;
	else
		Comment << "<none>";

	Comment << ";" << (100*theModExpRatio) << "%=lmt+";
	if (theModExpTime)
		 Comment << *theModExpTime;
	else
		Comment << "<none>";
		
}


/* VersionOpt */

VersionOpt::VersionOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr) {
}

bool VersionOpt::parse(const char *, const char *) {
	Assert(theCmdLine);
	cerr << PolyLetterhd() << endl;
	exit(0);
}

void VersionOpt::report() const {
	Comment << PolyVersion();
}
