
/*
 * $Id: AlarmClock.cc,v 1.2 1999/04/15 20:36:16 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include "Assert.h"
#include "AlarmClock.h"

AlarmClock TheAlarmClock;

/* Alarm */

void Alarm::ring() const {
	Assert(theSleeper);
	theSleeper->wakeUp(*this);
}

void Alarm::set(Time aTime, AlarmUser *aSleeper) {
	Assert(aSleeper);

	theSleeper = aSleeper;
	theTime = aTime;
}


/* AlarmUser */

AlarmUser::AlarmUser(): thePendAlarmCnt(0) {
}

AlarmUser::~AlarmUser() {
	Assert(!thePendAlarmCnt);
}

void AlarmUser::wakeUp(const Alarm &) {
	thePendAlarmCnt--;
}

void AlarmUser::sleepTill(Time time) {
	TheAlarmClock.setAlarm(Alarm(time, this));
	thePendAlarmCnt++;
}

void AlarmUser::sleepFor(Time delay) {
	sleepTill(TheClock + delay);
}


/* AlarmClock */

AlarmClock::AlarmClock(): theTimeLeftMin(0, 0) {
}

void AlarmClock::setAlarm(const Alarm &alarm) { 
	if (alarm) {
		theQueue.add(alarm);
		if (theRingTime <= 0 || alarm.time() < theRingTime)
			noteNewRingTime(alarm.time());
	}	
}

// returns time left (negative if late)
void AlarmClock::ring() {
	while (!theQueue.empty()) {
		if (Alarm &alarm = theQueue.top()) {
			noteNewRingTime(alarm.time());
			if (theTimeLeft > 0) // not ready yet
				break;
			alarm.ring();
		}
		theQueue.skip();
	}
}

void AlarmClock::update(Time curTime) {
	Clock::update(curTime);
	if (theRingTime <= curTime)
		ring();
}

void AlarmClock::noteNewRingTime(Time time) {
	theTimeLeft = time - theCurTime;
	theRingTime = time;
	if (theTimeLeftMin > theTimeLeft)
		theTimeLeftMin = theTimeLeft;
}
