
/*
 * $Id: AlarmClock.h,v 1.2 1999/04/15 20:36:17 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef ALARM_CLOCK_H
#define ALARM_CLOCK_H

#include "Heap.h"
#include "Clock.h"

class AlarmUser;

// one time alarm
class Alarm {
	public:
		Alarm(): theSleeper(0) {}
		Alarm(Time aTime, AlarmUser *aSleeper): theTime(aTime), theSleeper(aSleeper) {}

		operator void*() const { return theSleeper ? (void*)-1 : 0; }
		Time time() const { return theTime; }

		const AlarmUser *sleeper() const { return theSleeper; }

		void set(Time aTime, AlarmUser *aSleeper);
		void ring() const;
		void clear() { theSleeper = 0; }

		// these are used by AlarmQueue to order the alarms
		bool operator <(const Alarm &a) const { return theTime < a.theTime; }
		bool operator >(const Alarm &a) const { return theTime > a.theTime; }

	protected:
		Time theTime;
		AlarmUser *theSleeper;
};

// somebody who can wait for an alarm
// alarm count is used to check if pending alarms must be cleared 
// before object destruction
class AlarmUser {
	public:
		AlarmUser();
		virtual ~AlarmUser();

		virtual void wakeUp(const Alarm &alarm) = 0;  // but has a body!

	protected:
		virtual void sleepTill(Time time);
		void sleepFor(Time delay);

	protected:
		int thePendAlarmCnt;
};

// a sequence of ordered Alarms
typedef Heap<Alarm> AlarmQueue;


// alarm clock that can handle unlimited number of alarms
class AlarmClock: public Clock {
	public:
		AlarmClock();
		virtual ~AlarmClock() {}

		bool clear() const { return theQueue.empty(); }
		Time ringTime() const { return theRingTime; }
		Time timeLeft() const { return theTimeLeft; }
		Time timeLeftMin() const { return theTimeLeftMin; }

		virtual void update(Time curTime); // rings if needed
		void setAlarm(const Alarm &alarm);

	protected:
		void ring(); // wakes up ready sleepers
		void noteNewRingTime(Time time);

	protected:
		AlarmQueue theQueue;
		Time theRingTime;
		Time theTimeLeft;
		Time theTimeLeftMin;
};

extern AlarmClock TheAlarmClock;

#endif
