#ifndef X_ASSERT_H
#define X_ASSERT_H

/*
 * $Id: Assert.h,v 1.1.1.1 1999/01/27 08:40:45 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <errno.h>

class ostream;

// our version of assert(3), mostly for portability purposes
// this macro can be compile-time disabled for production runs
#ifndef NDEBUG
	#define Assert(cond) ((cond) ? (void)0 : Abort(__FILE__, __LINE__, #cond))
#else
	#define Assert(cond)
#endif

// same as Assert but calls Exit instead of Abort (and cannot be disabled)
#define Check(cond) ((cond) ? (void)0 : Exit(__FILE__, __LINE__, #cond))

// logs current error to cerr and exits if cond fails
#define Must(cond) ((cond) ? (void)0 : (Complain(__FILE__, __LINE__), ::exit(errno)))

// logs current error to cerr if cond fails
#define Should(cond) ((cond) ? true : (Complain(__FILE__, __LINE__), false))


// perror(3) wrapper (handy as a stream manipulator)
// note: str() method can be overwritten to support custom error messages
class error {
	public:
		error(int anErrNo): theErrNo(anErrNo) {}
		virtual ~error() {}

		int errNo() const { return theErrNo; }
		virtual const char *str() const;
		ostream &print(ostream &os) const;

	protected:
		int theErrNo;
};

inline 
ostream &operator <<(ostream &os, const error &e) { return e.print(os); }


// handy for temporary debugging
#define here __FILE__ << ':' << __LINE__ << ": "

/* internal functions used by macros above */

// logs current err to cerr
extern void Complain(const char *fname, int lineno);

// aborts program execution and generates coredump
extern void Abort(const char *fname, int lineno, const char *cond);

// aborts program execution without coredump
extern void Exit(const char *fname, int lineno, const char *cond);

#endif
