
/*
 * $Id: History.h,v 1.2 1999/04/15 20:36:19 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef HISTORY_CLASS_H
#define HISTORY_CLASS_H

#include "Array.h"
#include "gadgets.h"

// History log: a circular buffer; new entries overwrite old ones
// all external coordinates are relative to "now" or last entry
// for simplicity, assume that at most MAX_INT items are inserted
template <class Item>
class History {
	public:
		History(int aCapacity = 0): theLog(aCapacity), theOffset(0) {}

		void capacity(int aCapacity) { Assert(!theOffset); theLog.stretch(aCapacity); }

		int depth() const { return Min(theOffset, theLog.capacity()); }
		Item at(int off) const { return theLog[(theOffset - off) % theLog.capacity()]; }
		void insert(Item i) { theLog[++theOffset % theLog.capacity()] = i; }

	protected:
		Array<Item> theLog;
		int theOffset;      // beginning of a buffer
};

#endif
