
/*
 * $Id: rndDistrs.cc,v 1.1.4.1 1999/05/21 04:20:10 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include <iostream.h>

#include "Assert.h"
#include "rndDistrs.h"


UnifDistr::UnifDistr(RndGen *aGen, double aLo, double aHi):
	RndDistr(aGen), theLo(aLo), theHi(aHi) {

	Assert(theHi >= theLo);
}

ostream &UnifDistr::print(ostream &os) const {
	return os << pdfName() << '(' << theLo << ", " << theHi << ')';
}

double ExpDistr::trial() {
	return -theMean * log(theGen->trial());
}

double NormDistr::trial() {
	double sum = 0;

	for (int i = 0; i < 12; ++i)
		sum += theGen->trial();

	return theMean + theSDev*(sum - 6.0);
}

ostream &NormDistr::print(ostream &os) const {
	return os << pdfName() << '(' << theMean << ", " << theSDev << ')';
}

// translates mean and std dev into "internal" mu and sigma
// then creates the LognDistr object
LognDistr *LognDistr::ViaMean(RndGen *aGen, double aMean, double aSDev) {

	const double twoLnM = 2 * log(aMean);
	const double lnSum = log(aSDev*aSDev + aMean*aMean);

	const double aMu = twoLnM - lnSum/2;
	const double aSigmaSq = lnSum - twoLnM;

	return new LognDistr(aGen, aMu, aSigmaSq);
}

double LognDistr::trial() {
	return ::exp(NormDistr::trial());
}

// translates "internal" mu and sigma into mean
double LognDistr::mean() const {
	return ::exp(theMean + theSDev/2);
}

// translates "internal" mu and sigma into std dev
double LognDistr::sdev() const {
	const double w = ::exp(theSDev);
	return w * (w - 1) * ::exp(2*theMean);
}


/* Zipf */

// Euler-Mascheroni constant
static const double TheEMConst = 0.57721566490153286060651209;

double ZipfDistr::trial() {
	return floor(pow(theWorldCap+1, theGen->trial()));
}

int ZipfDistr::ltrial(int min, int max) {
	// wrong: (min, max]
	// return min + (int) floor(pow(max-min+1, theGen->trial()));
	// wrong: [min, max)
	// [min, max]
	min--;
	return min + (int) floor(pow(max-min+1, theGen->trial()));
}

double ZipfDistr::omega() const {
	// harmonic sum approximation: log(i) + gamma + 1/(2i)
	return log(theWorldCap) + TheEMConst + 1./(2*theWorldCap);
}

ostream &ZipfDistr::print(ostream &os) const {
	return os << pdfName() << '(' << theWorldCap << ')';
}


/* Seq */

SeqDistr::SeqDistr(RndGen *aGen, int aFirstId): RndDistr(aGen), 
	theFirstId(aFirstId), theLastId(aFirstId-1) {
}

double SeqDistr::trial() {
	return ++theLastId;
}

ostream &SeqDistr::print(ostream &os) const {
	return os << pdfName() << '(' << theFirstId << ')';
}
